package net.nevet5gi.buzzbot.modules;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;
import java.net.URLClassLoader;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.jar.Attributes;
import java.util.jar.JarInputStream;
import java.util.jar.Manifest;

public class ModuleLoader {
    private static final Logger LOGGER = LoggerFactory.getLogger(ModuleLoader.class);

    public static ArrayList<Path> getModules() {
        File modulesPath = new File("./modules/");
        if (!modulesPath.exists()) {
            modulesPath.mkdirs();
        }

        ArrayList<Path> modules = new ArrayList<>();
        try {
            Files.walk(Paths.get("./modules/"))
                    .filter(Files::isRegularFile)
                    .forEach(modules::add);
        } catch (IOException e) {
            System.out.println("Unable to get module list");
            e.printStackTrace();
        }

        return modules;
    }

    public static ArrayList<BuzzyModule> loadModules() throws IOException, ClassNotFoundException, NoSuchMethodException, InvocationTargetException, InstantiationException, IllegalAccessException {
        ArrayList<Path> modulePaths = getModules();

        ArrayList<BuzzyModule> modules = new ArrayList<>();
        for (Path module : modulePaths) {
            JarInputStream jarStream = new JarInputStream(new FileInputStream(String.valueOf(module)));
            Manifest mf = jarStream.getManifest();
            Attributes attributes = mf.getMainAttributes();
            String className = attributes.getValue("Main-Class");
            if (className == null) {
                LOGGER.error("Unable to get Main Class from module " + module.toString());
            }

            File file = module.toFile();
            URL[] urls = {file.toURI().toURL()};

            URLClassLoader urlClassLoader = URLClassLoader.newInstance(urls);

            if (urlClassLoader == null) {
                System.out.println("Classloader is null");
            }

            Class<?> mainClass = Class.forName(className, true, urlClassLoader);
            BuzzyModule newModule = (BuzzyModule) mainClass.getDeclaredConstructor().newInstance();
            newModule.onPreEnable();
            modules.add(newModule);
            if (jarStream != null ) {
                jarStream.close();
            }
            if (urlClassLoader != null) {
                urlClassLoader.close();
            }
        }
        return modules;
    }
}